<?php
// describes an editable list template that can be saved
// WPFB_ListTpl::Generate generates output for front-end file lists
class WPFB_ListTpl {
	
	var $tag;
	var $header;
	var $footer;
	var $file_tpl_tag;
	var $cat_tpl_tag;
	var $current_list = null;
        
		
	static function Get($tag) {
		$tag = trim($tag, '\'');
		$tpls = get_option(WPFB_OPT_NAME.'_list_tpls');
		return isset($tpls[$tag]) ? new WPFB_ListTpl($tag, $tpls[$tag]) : null;
	}
	
	static function GetAll() {
		$tpls = get_option(WPFB_OPT_NAME.'_list_tpls');
		foreach($tpls as $tag => $tpl)
			$tpls[$tag] = new WPFB_ListTpl($tag, $tpl);
		return $tpls;
	}
	
	function WPFB_ListTpl($tag=null, $data=null) {
		if(!empty($data)) {
			$vars = array_keys(get_class_vars(get_class($this)));
			foreach($vars as $var)
				if(isset($data[$var]))
					$this->$var = $data[$var];
		}				
		$this->tag = $tag;
	}
	
	function Save() {
		$tpls = get_option(WPFB_OPT_NAME.'_list_tpls');
		if(!is_array($tpls)) $tpls = array();
		$data = (array)$this;
		unset($data['tag'], $data['current_list']);
		$tpls[$this->tag] = $data; 
		update_option(WPFB_OPT_NAME.'_list_tpls', $tpls);
	}
	
	private function ParseHeaderFooter($str, $uid=null) {
		$str = preg_replace('/%sort_?link:([a-z0-9_]+)%/ie', __CLASS__.'::GenSortlink(\'$1\')', $str);
		
		if(strpos($str, '%search_form%') !== false)
			$str = str_replace('%search_form%', WPFB_Output::GetSearchForm("", $_GET), $str);
		
		$str = preg_replace('/%print_?script:([a-z0-9_-]+)%/ie', __CLASS__.'::PrintScriptOrStyle(\'$1\', false)', $str);
		$str = preg_replace('/%print_?style:([a-z0-9_-]+)%/ie', __CLASS__.'::PrintScriptOrStyle(\'$1\', true)', $str);
	
		if(empty($uid)) $uid = uniqid();
		$str = str_replace('%uid%', $uid, $str);
		
		
		$count = 0;
		$str = preg_replace("/jQuery\((.+?)\)\.dataTable\s*\((.*?)\)\s*;/", 'jQuery($1).dataTable(wpfb_DataTableOptionsFilter'.$uid.'($2));'."\r\n//%WPFB_DATA_TABLE_OPTIONS_FILTER%", $str, -1, $count);
		if($count > 0)
		{
			$dataTableOptions = array();
			list($sort_field, $sort_dir) = WPFB_Core::ParseFileSorting($this->current_list->file_order);			
			$file_tpl = WPFB_Core::GetTpls('file', $this->file_tpl_tag);
			if(($p = strpos($file_tpl, "%{$sort_field}%")) > 0)
			{
				// get the column index of field to sort
				$col_index = substr_count($file_tpl,"</t", 0, $p);				
				$dataTableOptions["aaSorting"] = array(array($col_index, strtolower($sort_dir)));
			}
			
			if($this->current_list->page_limit > 0)
					$dataTableOptions["iDisplayLength"] = $this->current_list->page_limit;
			
			
			$str = str_replace('//%WPFB_DATA_TABLE_OPTIONS_FILTER%', 
"
function wpfb_DataTableOptionsFilter{$uid}(options){
	var wpfbOptions = ".json_encode($dataTableOptions).";
	if('object' == typeof(options)) { for (var v in options) { wpfbOptions[v] = options[v]; } }
	return wpfbOptions;
}
", $str);
		}	
		
		return $str;
	}
	
	static function PrintScriptOrStyle($script, $style=false)
	{
		ob_start();
		if($style) wp_print_styles($script);
		else  wp_print_scripts($script);
		return ob_get_clean();
	}
	
	static function GenSortlink($by) {
		static $link;
		if(empty($link)) {
			$link = remove_query_arg('wpfb_file_sort');
			$link .= ((strpos($link, '?') > 0)?'&':'?').'wpfb_file_sort=&';	
		}
		$desc = !empty($_GET['wpfb_file_sort']) && ($_GET['wpfb_file_sort'] == $by || $_GET['wpfb_file_sort'] == "<$by"); 
		return $link.($desc?'gt;':'lt;').$by;
	}
	
	function GenerateList(&$content, $categories, $cat_grouping, $file_order, $page_limit, $cat_order=null )
	{
		$hia = WPFB_Core::GetOpt('hide_inaccessible');
		$sort = WPFB_Core::GetFileListSortSql($file_order);
		
		
		if($page_limit > 0) { // pagination
			$page = (empty($_REQUEST['wpfb_list_page']) || $_REQUEST['wpfb_list_page'] < 1) ? 1 : intval($_REQUEST['wpfb_list_page']);
			$start = $page_limit * ($page-1);
		} else $start = -1;
		
		if(!empty($_GET['wpfb_s'])) { // search
			wpfb_loadclass('Search');
			$where = WPFB_Search::SearchWhereSql(WPFB_Core::GetOpt('search_id3'), $_GET['wpfb_s']);
		} else $where = '1=1';
		
		$num_total_files = 0;
		if(is_null($categories)) { // if null, just list all files!
			$files = WPFB_File::GetFiles2($where, $hia, $sort, $page_limit, $start);
			$num_total_files = WPFB_File::GetNumFiles2($where, $hia);
				foreach($files as $file) $content .= $file->GenTpl2($this->file_tpl_tag);
		} else {
			if(!empty($cat_order))
				WPFB_Item::Sort($categories, $cat_order);
		
			$cat = reset($categories); // get first category
			if(count($categories) == 1 && $cat->cat_num_files > 0) { // single cat
				if(!$cat->CurUserCanAccess()) return '';
				if($cat_grouping) $content .= $cat->GenTpl2($this->cat_tpl_tag);
				$where = "($where) AND ".WPFB_File::GetSqlCatWhereStr($cat->cat_id);
				$files = WPFB_File::GetFiles2($where, $hia, $sort, $page_limit, $start);
				$num_total_files = WPFB_File::GetNumFiles2($where, $hia);

					 foreach($files as $file) $content .= $file->GenTpl2($this->file_tpl_tag);
			} else { // multi-cat
				// TODO: multi-cat list pagination does not work properly yet
		
				// special handling of categories that do not have files directly: list child cats!
				if(count($categories) == 1 && $cat->cat_num_files == 0) {
					$categories = $cat->GetChildCats(true, true);
					if(!empty($cat_order))
						WPFB_Item::Sort($categories, $cat_order);
				}
		
				if($cat_grouping) { // group by categories
					$n = 0;
					foreach($categories as $cat)
					{
						if(!$cat->CurUserCanAccess()) continue;
		
						$num_total_files = max($nf = WPFB_File::GetNumFiles2("($where) AND ".WPFB_File::GetSqlCatWhereStr($cat->cat_id), $hia), $num_total_files); // TODO
		
						//if($n > $page_limit) break; // TODO!!
						if($nf > 0) {
							$files = WPFB_File::GetFiles2("($where) AND ".WPFB_File::GetSqlCatWhereStr($cat->cat_id), $hia, $sort, $page_limit, $start);                                                     
							if(count($files) > 0) {
								$content .= $cat->GenTpl2($this->cat_tpl_tag); // check for file count again, due to pagination!
								foreach($files as $file) $content .= $file->GenTpl2($this->file_tpl_tag); 
							}
						}
					}
				} else {
					// this is not very efficient, because all files are loaded, no pagination!
					$all_files = array();
					foreach($categories as $cat)
					{
						if(!$cat->CurUserCanAccess()) continue;
						$all_files += WPFB_File::GetFiles2("($where) AND ".WPFB_File::GetSqlCatWhereStr($cat->cat_id), $hia, $sort);
					}
					$num_total_files = count($all_files);

					{
						 WPFB_Item::Sort($all_files, $sort);

						 $keys = array_keys($all_files);
						 if($start == -1) $start = 0;
						 $last = ($page_limit > 0) ? min($start + $page_limit, $num_total_files) : $num_total_files;

						 for($i = $start; $i < $last; $i++)
							  $content .= $all_files[$keys[$i]]->GenTpl2($this->file_tpl_tag);
					 }
				}
			}
		}
		
		return $num_total_files;
	}

        
	function Generate($categories, $cat_grouping, $file_order, $page_limit, $cat_order=null, $hide_pagenav = false)
	{
		$this->current_list = (object)compact('cat_grouping', 'file_order', 'page_limit', 'cat_order');
		
		// self::ParseFileSorting($sort, $attach_order);
		
		$uid = uniqid();
      
		
		$content = $this->ParseHeaderFooter($this->header, $uid);
		
		$num_total_files = $this->generateList($content, $categories, $cat_grouping, $file_order, $page_limit, $cat_order);
		
		$footer = $this->ParseHeaderFooter($this->footer, $uid);		
		$is_datatable = strpos($footer, ").dataTable(")!==false;
		
		// TODO: no page_limit when dataTable?
		// hide pagenav when using datatable
		$hide_pagenav = $hide_pagenav || $is_datatable;
		
		$page_break = $page_limit > 0 && $num_total_files > $page_limit;
		
		if($page_break && !$hide_pagenav) {
			$pagenav = paginate_links( array(
				'base' => add_query_arg( 'wpfb_list_page', '%#%' ),
				'format' => '',
				'total' => ceil($num_total_files / $page_limit),
				'current' => empty($_GET['wpfb_list_page']) ? 1 : absint($_GET['wpfb_list_page'])
			));

			if(strpos($footer, '%page_nav%') === false)
				$footer .= $pagenav;
			else
				$footer = str_replace('%page_nav%', $pagenav, $footer);
		} else {
			$footer = str_replace('%page_nav%', '', $footer);
		}
		
		
		$content .= $footer;
		
		return $content;
	}
	
	function Sample($cat, $file) {
		$uid = uniqid();
		$this->current_list = (object)array('cat_grouping' => false, 'file_order' => null, 'page_limit' => 3, 'cat_order' => null);
	
		$footer = str_replace('%page_nav%', paginate_links(array(
			'base' => add_query_arg( 'wpfb_list_page', '%#%' ), 'format' => '',
			'total' => 3,
			'current' => 1
		)), $this->ParseHeaderFooter($this->footer, $uid));
		return $this->ParseHeaderFooter($this->header, $uid) . $cat->GenTpl2($this->cat_tpl_tag) . $file->GenTpl2($this->file_tpl_tag) . $footer;		
	}
	
	function Delete() {
		$tpls = get_option(WPFB_OPT_NAME.'_list_tpls');
		if(!is_array($tpls)) return;
		unset($tpls[$this->tag]);
		update_option(WPFB_OPT_NAME.'_list_tpls', $tpls);
	}
	
	function GetTitle() { return __(__(esc_html(WPFB_Output::Filename2Title($this->tag))), WPFB); }
}